% assemble_cov:
%
% Return the covariance matrix that is implied by a particular
% choice of z_array. Remember that z_array is indexed according to
% 'subset', *not* according to the global set of variables
% (hence, elements are accessed as 'z_array(i)', *not*
% 'z_array(subset(i))'.
%
% Author: Ricardo Silva (ricardo@stats.ucl.ac.uk)
% Last modification: Jan 21 2009
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Inputs:
%
% - z_array: an instantiation of the mixture indicator vector.
%
% Outputs:
%
% - output_cov: the corresponding covariance matrix according
%               to the mixture model.

function [output_cov] = assemble_cov(subset, z_array)

global vars; global covs; global graph;

n = length(subset);
output_cov = zeros(n, n);

for i = 1:n
    for j = (i + 1):n        
        if graph(subset(i), subset(j)) == 0
           continue
        end
        v = get_parameter_index(subset(i), subset(j), z_array(i), z_array(j));
        output_cov(i, j) = covs{subset(i), subset(j)}(v);
        output_cov(j, i) = output_cov(i, j);
    end
    v = z_array(i);
    output_cov(i, i) = vars{subset(i)}(v);
end
