% assemble_mean:
%
% Return the mean vector that is implied by a particular
% choice of z_array. Remember that z_array is indexed according to
% 'subset', *not* according to the global set of variables
% (hence, elements are accessed as 'z_array(i)', *not*
% 'z_array(subset(i))'.
%
% Author: Ricardo Silva (ricardo@stats.ucl.ac.uk)
% Last modification: Jan 21 2009
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Inputs:
%
% - z_array: an instantiation of the mixture indicator vector.
%
% Outputs:
%
% - output_mean: the corresponding means for each variable according
%                to the mixture model.

function [output_mean] =  assemble_mean(subset, z_array)

global means;

n = length(subset);
output_mean = zeros(n, 1);

for i = 1:n
    v = z_array(i);
    output_mean(i) = means{subset(i)}(v);
end
