% graphLearning:
%
% Learn the structure of a graph by using Arthur Gretton's marginal
% independence test. Please go to this homepage and download the package
% http://www.kyb.mpg.de/bs/people/arthur/indep.htm
%
% Input: the data matrix and a significance level 'alpha' for the test.
% In our paper, I used 0.05 (rather arbitrarily). An alternative, more
% powerful (?) but slower (?) test within Arthur's package might provide
% better results (see his documentation).

function graph = graph_learning(data, alpha)

params.sigx = -1;
params.sigy = -1;

num_vars = size(data, 2);
graph = zeros(num_vars, num_vars);

for i = 1:num_vars
    X = data(:, i);
    for j = (i + 1):num_vars
        Y = data(:, j);
        [thresh,testStat,params] = hsicTestGamma(X, Y, alpha, params);
        if testStat > thresh
            disp(sprintf('Adding edge (%d, %d)', i, j));
            graph(i, j) = 1;
            graph(j, i) = 1;
        end
    end
end
