function [clique_matrix,num_cliques]=find_cliques(adj,names,org_size)
%finds the cliques in an adjacency matrix if running set org_size to -1
% this program is really ineffecient and runs recursively Version 1.0
%
%The algortithm works by ordering the nodes by their degree
%It chooses the higest degree node-node
%then reduces the adjacency matrix to nodes that are adjacent to node
%then finds cliques in the reduced matrix
%any cliques found also cliques with node added
%if there are no adjacent nodes node is a clique by itself
%then node is removed from the matrix
% and cliques found in this reduced matrix
%if any of these cliques are not dominated by cliques already found they are added
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if org_size<0
	%first time it is run
	org_size=size(adj,1);
	names=[1:org_size];
end

num_cliques=0;
clique_matrix=zeros(org_size,1)';

%reorder the adjacency matrix by the degree of the nodes
[adj,names]=reorder_adjacency(adj,names);

%what is the highest degree node
node=names(size(names,2));

% then find the submatrix of only the adjacent nodes to the highest degree node
[reduced_adj,reduced_names,degree]=find_adjacent_nodes(adj,names);

if degree>0
	%there will be at least one clique
	%find them and report back
	%disp('first half-about to call function recursively');
	%node
	%reduced_adj
	%reduced_names
	%org_size
	%pause
	[temp_clique_matrix,add]=find_cliques(reduced_adj,reduced_names,org_size);
	%temp_clique_matrix
	%add these cliques to the list
	for i=1:add
		num_cliques=num_cliques+1;
		clique_matrix(num_cliques,:)=temp_clique_matrix(i,:);
		clique_matrix(num_cliques,node)=1;
	end
else
	%do not return the first clique ie the zeros
	num_cliques=num_cliques+1;
	clique_matrix(num_cliques,node)=1;
end

% remove the node and find more cliques
if size(adj,1)>1
	new_adj=adj(1:size(adj,1)-1,1:size(adj,1)-1);
	new_names=names(1:size(adj,1)-1);

	%find them and report back
	%disp('second half-about to call function recursively');
	%new_adj
	%new_names
	%org_size
	%pause
	[temp_clique_matrix,add]=find_cliques(new_adj,new_names,org_size);
	%temp_clique_matrix
	old_num_cliques=num_cliques;
	%add these cliques to the list
	for i=1:add
		% check if new clique is dominated by others already added
		total=sum(temp_clique_matrix(i,:));
		if max(clique_matrix(1:old_num_cliques,:)*temp_clique_matrix(i,:)')<total
			num_cliques=num_cliques+1;
			clique_matrix(num_cliques,:)=temp_clique_matrix(i,:);
		end
	end
end
%clique_matrix(:,1:14)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [new_adj,new_names]=reorder_adjacency(adj,names)
% takes an adjacency matrix and a name vector and will reorder both to sorted by the largest degree.
row_sum=sum(adj);
temp=sortrows([row_sum',names',adj]);
%size(temp)
new_names=temp(:,2)';
new_adj=temp(:,[3:size(adj,2)+2])';
%size(new_adj)
temp=sortrows([row_sum',names',new_adj]);
new_adj=temp(:,[3:size(adj,2)+2]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [new_adj,new_names,degree]=find_adjacent_nodes(adj,names)
%This function returns the adjacency matrix of the nodes  that are adjacent to last node in the matrix
degree=0;
new_adj=[];
new_names=[];

for i=1:size(adj,1)-1
	if adj(size(adj,1),i)==1
		degree=degree+1;
		temp_adj(degree,:)=adj(i,:);
		new_names(degree)=names(i);
		cols(degree)=i;
	end
end

if degree>0
	for i=1:degree
		new_adj(:,i)=temp_adj(:,cols(i));
	end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%55


%%%%%%%%%%%%%%%%%%%%%%%
%Copyright (c) 2005, Stuart Mitchell
%All rights reserved.
%
%Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
%
%    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
%    * Neither the name of the University of Auckland nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
%
%THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
%%%%%%%%%%%%%%%%%%%

