% learn_means:
%
% Fit the means of Y_i making use of the current Bartlett parameters.
%
% Author: Ricardo Silva (ricardo@stats.ucl.ac.uk)
% Last modification: Oct 08 2008
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Inputs:
%
% - i: target variable
% - b, gamma: current Bartlett parameters

function [new_means] = learn_means(i, subset, b, gamma)

global means;
global data;
global z_post;

[maximal_z z_universe] = get_z_range(subset);

num_means = length(means{i});
new_means = zeros(num_means, 1);
denominator = zeros(num_means, 1);
sample_size = size(data, 1);
n = length(subset);

impl_mean = zeros(n, maximal_z);
for zi = z_universe
    current_z = get_z(subset, zi);
    impl_mean(:, zi) = assemble_mean(subset, current_z);    
end

i_pos = find(subset == i);
other_pos = setdiff(1:n, i_pos);

if n > 1
    for d = 1:sample_size
        for zi = z_universe
            current_z = get_z(subset, zi);
            v = current_z(i_pos);
            ratio_d = z_post(d, zi) / gamma(zi);
            denominator(v) = denominator(v) + ratio_d;
            residuals = data(d, subset) - impl_mean(:, zi)';
            residuals = residuals(other_pos);
            new_means(v) = new_means(v) + ratio_d * (data(d, i) - residuals * b(:, zi));
        end
    end
else
    for d = 1:sample_size
        for zi = z_universe
            current_z = get_z(subset, zi);
            v = current_z(i_pos);
            ratio_d = z_post(d, zi) / gamma(zi);
            denominator(v) = denominator(v) + ratio_d;
            new_means(v) = new_means(v) + ratio_d * data(d, i);
        end
    end
end

new_means = new_means ./ denominator;
if sum(isnan(new_means)) > 0
    error('means!');
end
