% local_expectation:
%
% The 'expectation' step of the EM algorithm: cache all posterior
% distributions for all possible z instantiations and data points,
% within the 'subset' of variables only.
%
% Needless to say, this is very memory and cpu intense for large subsets.
%
% Author: Ricardo Silva (ricardo@stats.ucl.ac.uk)
% Last modification: Oct 08 2008
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [cached_z] = local_expectation(subset)

global data;
global pi_z;

[maximal_z z_universe] = get_z_range(subset);

sample_size = size(data, 1);
n = length(subset);
cached_z = zeros(sample_size, maximal_z);

for zi = z_universe
    current_z = get_z(subset, zi);
    log_prob_zi = 0;
    for i = 1:n
        log_prob_zi = log_prob_zi + log(pi_z{subset(i)}(current_z(i)));
    end
    impl_cov  = assemble_cov(subset, current_z);
    impl_mean = assemble_mean(subset, current_z);
    inv_impl  = inv(impl_cov);
    if det(impl_cov) < 0
        error('local_expection: NeG here!');
    end
    logdet = log(det(impl_cov));
    if (isnan(logdet))
        error('local_expection: NaN here!');
    end
    for d = 1:sample_size
        dmean = data(d, subset)' - impl_mean;
        cached_z(d, zi) = -0.5 * (logdet + trace(inv_impl * (dmean * dmean'))) + log_prob_zi;
    end
end

for d = 1:sample_size
    cached_z(d, :) = cached_z(d, :) - max(cached_z(d, :));
    cached_z(d, :) = exp(cached_z(d, :));
    cached_z(d, :) = cached_z(d, :) / sum(cached_z(d, :));
end
