%EM score: takes as input the posterior distribution of z's given
%previous parameter instantiation

function [score] = lvm_em_score()
                        
global data;    
global z_posterior;
global pi_z;
global z_universe;

score       = 0;
sample_size = size(data, 1);
num_vars    = size(data, 2);

%Likelihood term
for zi = z_universe
    current_z = get_z(1:num_vars, zi);    
    if prob_zero(1:num_vars, current_z) == 0
        continue;
    end;
    impl_cov    = lvm_assemble_cov(current_z);
    impl_mean_t = lvm_assemble_mean(current_z)';
    inv_impl    = inv(impl_cov);
    logdet      = log(det(impl_cov));
    
    sum_p = 0.;
    mean_buffer_m = zeros(num_vars, num_vars);
    for d = 1:sample_size
        p_z = z_posterior(d, zi);
        sum_p = sum_p + p_z;
        mean_buffer = data(d, :) - impl_mean_t;
        mean_buffer_m = mean_buffer_m + p_z * mean_buffer' * mean_buffer;
        for c = 1:num_vars
            score = score + log(pi_z{c}(current_z(c))) * p_z;
            if (isnan(score)) 
                error('EMScore == NaN');
            end
        end
    end
    
    score = score - 0.5 * ((logdet + num_vars * 2 * pi) * sum_p + trace(inv_impl * mean_buffer_m));
    if (isnan(score)) 
        error('EMScore == NaN');
    end
end

score = score / sample_size;

