% learn_lvm: 
%
% Fit a mixture model defined by the canonical latent variable model.
%
% Author: Ricardo Silva (ricardo@stats.ucl.ac.uk)
% Last modification: Oct 07 2008
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% For details in inputs and outputs, see 'learn.m'

function [intercepts_out, coeffs_out, gammas_out, pi_z_out] = ...
         lvm_learn(graph_in, data_in, num_components_in, T, num_trials, tol)

% Global variable declaration

global pi_z;
global graph;   global data;
global spouses; global non_spouses;
global num_components;
global maximal_z; global z_universe;
global z_posterior; global l_posterior; global l2_posterior;

global score;
global cliques; global intersections; global clique_parents;
global intercepts; global coeffs; global gammas;

% Initialization

data = data_in;
graph = graph_in;
num_components = num_components_in;
num_vars = length(graph);

[cliques, intersections, clique_parents] = build_clique_information(graph);

maximal_z = 1;
for i = 1:num_vars
    maximal_z = maximal_z * num_components(i);
end
z_universe = 1:maximal_z;

spouses     = generate_spouses(graph);
non_spouses = generate_nonspouses(graph);

best_score = -Inf;

% Maximum likelihood: iterations

for trial = 1:num_trials
    
    if (num_components > 1)
       disp(sprintf('Latent variable fitting, Initialization # %d', trial));
    end
    
    [intercepts, coeffs, gammas, pi_z] = lvm_create_params(data, graph);

    for t = 1:T
        if (num_components > 1)
           disp(sprintf('--------------------------ITERATION: %d', t));
        end
        z_posterior = lvm_expected_z();
        [l_posterior l2_posterior] = lvm_expectation_l(); 

        old_score = score;
        score = lvm_em_score();    
        disp(sprintf('EM score: %f', score));
        if abs(score - old_score) < tol
            break;
        end

        coeffs = lvm_optimize_coeffs(); 
        gammas = lvm_optimize_gammas();    
        break_loop = false;
        for i = 1:num_vars
            if ~isempty(gammas{i}(gammas{i} < 0.000001))
                break_loop = true;
                break;
            end
        end
        if break_loop
           break
        end
        intercepts = lvm_optimize_means();
        old_score = score;
        score = lvm_em_score();
        if (num_components > 1)
           disp(sprintf('EM score for iteration: %f', score));
        end
        if score < old_score
            disp('!'); %numerical rounding problems detected
        end
        if abs(score - old_score) < 0.0001
            if (num_components > 1)
               disp(sprintf('Final EM score: %f', score));
            end
            break;
        end

        %learn pZ

        pi_z = lvm_learn_pz();        
    end

    if score > best_score
        disp(sprintf('****** SCORE IMPROVEMENT: %f --> %f', best_score, score));
        intercepts_best = intercepts;
        coeffs_best = coeffs;
        gammas_best = gammas;
        pi_z_best = pi_z;
        best_score = score;
    end
end

intercepts_out = intercepts_best; 
coeffs_out = coeffs_best; 
gammas_out = gammas_best; 
pi_z_out = pi_z_best;

% Clear variables

clear pi_z;
clear graph;   clear data;
clear spouses; clear non_spouses;
clear num_components; clear maximal_z; clear z_universe;
clear z_posterior;
clear res_params;
clear cliques; clear intersections; clear clique_parents;
clear intercepts; clear coeffs; clear gammas;
clear l_posterior; clear l2_posterior;
