% lvm_predictive_llik:
% predictive log-likelihood on testset 'test'

function result = lvm_predictive_llik(test, interceptsI, coeffsI, gammasI, pi_zI, graphI, num_componentsI)

disp('Calculating predictive log-likelihood (latent variable model)...');

global intercepts; global coeffs; global gammas; global pi_z;
intercepts = interceptsI; coeffs = coeffsI; gammas = gammasI; pi_z = pi_zI;

global graph;   global data; global num_components;
graph = graphI; data = test; num_components = num_componentsI;

global spouses; global non_spouses;
global maximal_z;

num_vars = length(graph);
test_size = size(test, 1);

[maximal_z z_universe] = get_z_range(1:num_vars);
spouses = generate_spouses(graph);
non_spouses = generate_nonspouses(graph);

result = 0;

global cliques; global intersections; global clique_parents;
[cliques, intersections, clique_parents] = build_clique_information(graph);

for d = 1:test_size
    pred = zeros(maximal_z, 1);
    prob = zeros(maximal_z, 1);
    for zi = z_universe
        current_z = get_z(1:num_vars, zi);

        prob(zi) = 1;
        for c = 1:num_vars
            prob(zi) = prob(zi) * pi_z{c}(current_z(c));
        end    
        if prob(zi) == 0
            pred(zi) = -Inf;
            continue;
        end
        
        impl_mean = lvm_assemble_mean(current_z);
        impl_cov  = lvm_assemble_cov(current_z);
        impl_det  = det(impl_cov);
        logdet = log(impl_det);
        inv_impl = inv(impl_cov);    
        dmean = test(d, :)' - impl_mean;
        
        pred(zi) = log(prob(zi)) -0.5 * (num_vars * log(2 * pi) + ...
                   logdet + trace(inv_impl *(dmean * dmean')));
    end    

    pred = pred(prob > 0);
    max_pred = max(pred);
    pred = pred - max_pred; 
    result = result + log(sum(exp(pred))) + max_pred;
end

result = result / test_size;
