% build_gamma_mapping:
%
% Defines a linear mapping between a basis vector of gamma parameters
% (the reference classes) and all gamma parameters. Returns also the
% indices of the gammas selected as a basis.
%
% Author: Ricardo Silva (ricardo@stats.ucl.ac.uk)
% Last modification: Oct 07 2008
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Inputs:
%
% - i: target variable

function [M bM free_index] = build_gamma_mapping(i, subset, b)

global pi_z;
global covs;
global graph;

[maximal_z z_universe] = get_z_range(subset);
i_pos = find(subset == i);

%For each assignment of the parent cliques
num_v = length(pi_z{i});
M = sparse(maximal_z, num_v);
bM = zeros(maximal_z, 1);
num_free = num_v;
free_index = zeros(num_free, 1);
free_count = 1;
other_subset = setdiff(subset, i);

for v = 1:num_v
   
    %Choose reference class, get all offsets
    all_offsets = -Inf * ones(maximal_z, 1);
    for zi = z_universe
       z_inst = get_z(subset, zi);
       if z_inst(i_pos) ~= v
          continue
       end
       all_offsets(zi) = 0;
       for j = other_subset
           j_other_pos = find(other_subset == j);
           if graph(i, j) == 0
              continue
           end
           z_inst = get_z(subset, zi);
           j_pos = find(subset == j);
           pi = get_parameter_index(i, j, z_inst(i_pos), z_inst(j_pos));
           all_offsets(zi) = all_offsets(zi) + b(j_other_pos, zi) * covs{i, j}(pi);
       end
    end
    [reference_offset z_refp] = max(all_offsets); 
    free_index(free_count) = z_refp;
    free_count = free_count + 1;
    
    %Iterate over all z configurations with the same v assignment
    for zi = z_universe
        z_inst = get_z(subset, zi);
        if z_inst(i) ~= v
           continue;
        end
        %agreement found
        M(zi, v) = 1;
        bM(zi) = reference_offset - all_offsets(zi);
    end       
        
end

