% maximize_b_analytical:
%
% Find the conditional maximum likelihood for Bartlett coefficients
% 'b' as if there was no quadratic constraints.

function new_b = maximize_b_analytical(i, subset, gamma, b_param, fixed)

global data;
global z_post;

sample_size = size(data, 1);
n = length(subset);
[maximal_z z_universe] = get_z_range(subset);
num_params = (n - 1) * maximal_z;
i_pos = find(subset == i);
others = setdiff(subset, i);
num_coeff = n - 1;
other_pos = zeros(n - 1, 1);
for v = 1:(n - 1)
    other_pos(v) = find(subset == others(v));
end

% Find 'mu' and 'A', parameters of the quadratic function over b

mu = sparse(num_params, 1);
A = zeros(num_params, num_params);

for zi = z_universe
    current_z = get_z(subset, zi);    
    if prob_zero(subset, current_z) == 0
       continue;
    end;
    impl_mean = assemble_mean(subset, current_z);
    y_center = data(:, i) - impl_mean(i_pos);
    mean_others = impl_mean(other_pos)';
    x_center = data(:, others) - mean_others(ones(1, sample_size), :);
    start_p = (zi - 1) * num_coeff + 1;
    end_p = zi * num_coeff;
    yz_center = z_post(:, zi) .* y_center;
    mu(start_p:end_p) = sum(yz_center(:, ones(1, num_coeff)) .* x_center, 1) / gamma(zi);
    for d = 1:sample_size
        A(start_p:end_p, start_p:end_p) = A(start_p:end_p, start_p:end_p) +...
                z_post(d, zi) * x_center(d, :)' * x_center(d, :);
    end
    A(start_p:end_p, start_p:end_p) = A(start_p:end_p, start_p:end_p) / gamma(zi);
end

A = 0.5 * (A + A'); % Correcting rounding errors

% Find 'C', derivatives of the constraints with respect to the parameters

[Aeq1 beq1] = get_fixing_cov_constraints(i, subset, fixed);
[Aeq2 beq2] = get_mixture_cov_constraints(i, subset); 
Aeq  = [Aeq1; Aeq2];
beq  = [beq1; beq2];

% 'Aeq' Includes coefficients and variances, so let's select only variances

col_sel = ones(n * maximal_z, 1);
for zi = z_universe
    col_sel(zi * n) = 0;
end
C      = Aeq(:, col_sel == 1);
C_comp = Aeq(:, col_sel == 0);
beq    = beq - C_comp * gamma;

options = optimset('Display', 'off');
new_b = quadprog(A, -mu, [], [], C, beq, [], [], b_param, options);

